
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import re
from urlparse import urlparse

class Action:
    'Action base'
    
    realaction = False    

    def generate_header(self, writer, text=None):
        if text is None:
            if self.realaction:
                writer.action_nr += 1
                text = 'Auto-generated action #%d' % writer.action_nr
            else:
                text = 'Auto-generated action'
        writer.write('# %s' % (text, ))
        writer.newline()

    def generate_footer(self, writer):
        writer.newline()

    re_modifier = re.compile('(.+):([^:]+)$')
    re_record = re.compile('([^:]+)\.([^\.:]+)[^\.]*$')
    re_plain = re.compile('([^:]+)[^\.]*$')
    
    def gen_localvar_line(self, writer, vname, text):
        # the localvar part will only be emitted if it is not in the local vars yet
        if not vname in writer.localvars:
            writer.write(text)
            writer.newline()
            writer.localvars.append(vname)

    def generate_find_control(self, writer, name, index):
        # try to generate new-style 
        # GENERATES some prefix, and RETURNS the result! (that can be embedded into a line)
        if not writer.oldstyle:
            match = self.re_modifier.match(name)
            if match:
                firstpart, modifier = match.groups()
                if modifier == 'records':
                    match = self.re_record.match(firstpart)
                    if match:
                        rname, rpart = match.groups()
                        self.gen_localvar_line(writer, rname, '%s = %s.find_zope_records_control("%s")' % (rname, writer.browservar, rname))
                        return '%s[%d]["%s"]' % (rname, index, rpart)
                elif modifier == 'record':
                    match = self.re_record.match(firstpart)
                    if match:
                        rname, rpart = match.groups()
                        # Only good if there was no index.
                        if index == 0:
                            self.gen_localvar_line(writer, rname, '%s = %s.find_zope_record_control("%s")' % (rname, writer.browservar, rname))
                            return '%s["%s"]' % (rname, rpart)
                elif modifier == 'list':
                    match = self.re_plain.match(firstpart)
                    if match:
                        (lname, ) = match.groups()
                        self.gen_localvar_line(writer, lname, '%s = %s.find_zope_list_control("%s")' % (lname, writer.browservar, lname))
                        return '%s[%d]' % (lname, index)
                
        # fallback: plain old style
        retval = '%s.find_control(name="%s"' % (writer.browservar, name)
        # nr can be omitted if zero
        if index > 0:
            retval += ', nr=%s' % (index, )
        retval += ')'
        return retval

    def gen_urlpath(self, writer, path_info):
        'urlpath generation part.'
        path_info = urlparse(path_info)[2]
        if writer.re_path_within_portal:
            match = writer.re_path_within_portal.match(path_info)
        else:
            match = None
        if match:
            (path_within_portal, ) = match.groups()
            text = 'portal_url + "%s"' % (path_within_portal, )
        else:
            text = 'root_url + "%s"' % (path_info, )
        writer.write(text)

class SelectFormMixin:
    
    def set_form_data(self, all_forms, nr):
        '''Sets the form data

        all_forms is all the forms on the page
        nr is the index of this form.
        '''
        self.b_form = b_form = all_forms[nr]
        forms_with_this_name = [form for form in all_forms if form.name == b_form.name]
        if len(forms_with_this_name) != 1:
            self.form_nr_within_name = forms_with_this_name.index(b_form)
        else:
            self.form_nr_within_name = None
        self.form_name = b_form.name
        self.form_nr = nr

    def gen_select_form(self, writer, do_remark=False):
        '''Generates the select_form statement.

        If the form name is unique, use the name.

        If not unique, use name and number.

        If there is no form name at all (=None), use only number.

        "use_remark" can be specified to spit out a remark on the
        end of the line.
        '''
        remark = None
        if not self.form_name:
            # if there was no form name, find by number
            formsearch = 'nr=%d' % self.form_nr
            if do_remark:
                remark = 'Unnamed form'
        else:
            # use form name
            if do_remark:
                remark = 'Form %s' % (repr(self.form_name), )
            if self.form_nr_within_name is None:
                # only one form by this name, find by name only
                formsearch = 'name="%s"' % (self.form_name, )
            else:
                # find by name and nr
                formsearch = 'name="%s", nr=%d' % (self.form_name, self.form_nr)
        writer.write(writer.add_remark('%s.select_form(%s)' % (writer.browservar, formsearch), remark))
        writer.newline()
