        
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Form check

Generation of checking the values of the controls in the form
'''

from base import SelectFormMixin, Action
from Products.mechagen.generator.utils import counter

class CheckItem:
    'Check item in the submit action'

    def __init__(self, name, index, value, remark, commented=False):
        self.name = name
        self.index = index
        self.value = value
        self.remark = remark
        self.commented = commented

    def generate(self, writer, action):
        control = action.generate_find_control(writer, self.name, self.index)
        # generate diff
        valuetxt = '%s.value' % control
        value2txt = repr(self.value)
        if writer.root_url in value2txt:
            # remark the whole line in case the portal host and port appear
            # because this would break the test for sure (e.g. referer info)
            self.commented = True
        self._generate_diff(writer, valuetxt, value2txt)

    def _generate_diff(self, writer, v1, v2):
        if self.commented:
            # comment the line if necessary
            writer.write('# ')
        text = 'self.assertEqual(%s, %s)' % (v1, v2)
        if self.remark is not None:
            text = writer.add_remark(text, self.remark)
        writer.write(text)
        writer.newline()

class CheckListingItem(CheckItem):
    'Check item for checkboxes and selects'

    def __init__(self, name, index, value, possible_items, remark=None, commented=False):
        self.name = name
        self.index = index
        self.value = value
        self.possible_items = possible_items
        self.remark = remark
        self.commented = commented

    def generate(self, writer, action):
        control = action.generate_find_control(writer, self.name, self.index)
        # generate diff
        valuetxt = '%s.value' % control
        value2txt = repr(self.value)
        if writer.root_url in value2txt:
            # remark the whole line in case the portal host and port appear
            # because this would break the test for sure (e.g. referer info)
            self.commented = True
        self._generate_diff(writer, valuetxt, value2txt)
        # generate choice diffs
        ##valuetxt = '%s.possible_items()' % control   # Deprecated
        valuetxt = '[item.name for item in %s.items]' % control
        value2txt = repr(self.possible_items)
        self._generate_diff(writer, valuetxt, value2txt)

class FormCheck(SelectFormMixin, Action):
    'Check the equality of the form vars within one form'

    def __init__(self, all_forms, nr):
        self.set_form_data(all_forms, nr)
        formvarcounter = counter()
        checks = self.checks = []
        for control in self.b_form.controls:
            # get the index of the elem with the same name
            name = control.name
            index = formvarcounter.get_count(name)
            # only consider given types, and ones with an existing name.
            if name is not None:
                if control.type in ['checkbox', 'radio', 'select']:
                    # List-style controls:
                    # both value and choices are checked.
                    checks.append(CheckListingItem(name, index, control.value, 
                        [item.name for item in control.items], remark=None))
                        ##control.possible_items(), remark=None)) # Deprecated
                #some types are ignored altogether
                elif control.type in ['text', 'password', 'hidden', 'textarea',
                    'file', # ???
                    ]:
                    # Simple controls:
                    # mark hidden items
                    if control.type in ['hidden']:
                        remark = control.type.upper()
                    else:
                        remark = None
                    # check the value only
                    checks.append(CheckItem(name, index, control.value, remark))
            # count one
            formvarcounter.set_count(name)
            
    def generate(self, writer):
        writer.write('# Form %s' % (repr(self.form_name), ))
        writer.newline()
        # forms are selected by nr because some have no name
        self.gen_select_form(writer)
        # allow local var handling in this part.
        writer.localvars = []
        for check in self.checks:
            check.generate(writer, self)
        writer.localvars = []
