
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Plugin registration
'''

try:
    import BeautifulSoup
except ImportError:
    # XXX log this to Zope? --- "Plugins disabled without beautifulsoup"
    BeautifulSoup = None
else:
    Base = BeautifulSoup.BeautifulSoup
    class BeautifulSoupBase(Base, object):

        # Do something with tables or they cannot be nested.
        # The original parser closes the first table before starting the
        # new one. This should not be since we have nested tables.
        # (The below change is sufficient, but maybe too much?)
        
        RESET_NESTING_TAGS = Base.RESET_NESTING_TAGS
        RESET_NESTING_TAGS['table'] = []
        
        NESTABLE_TAGS = Base.NESTABLE_TAGS
        NESTABLE_TAGS['table'] = []
    
        # Make Null available on the class
        Null = BeautifulSoup.Null
        
registered_plugins = []
beautifulsoup_class = None

def register_plugins(plugins):
    'Registers a list of plugins'
    global registered_plugins
    global beautifulsoup_class
    if BeautifulSoup is None:
        return
    klasses = []
    for plugin in plugins:
        if plugin not in registered_plugins:
            registered_plugins.append(plugin)
    klasses = [p.BeautifulSoupMixin for p in registered_plugins]
    bases = tuple(klasses) + (BeautifulSoupBase, )
    beautifulsoup_class = type('BeautifulSoup parser', bases, {})

def get_plugins():
    return registered_plugins

def get_beautifulsoup_class():
    'Returns the soup class or None if plugins are disabled'
    return beautifulsoup_class
